/** @file init.c
 * @brief R specific functions.
 * @date 2010-03-04
 * @author Łukasz Bartnik
 */

#include <Rinternals.h>
#include <Rdefines.h>
#include <R_ext/Rdynload.h>

#include <string.h>
#include "wrapper.h"

static char input_data_path[1024] = {0};

// --------------------------------------------------------------------

#define RETURN_BOOL(val) do {       \
		SEXP ret = NEW_LOGICAL(1);  \
		LOGICAL_DATA(ret)[0] = val; \
		return ret;                 \
	} while (0);
#define RETURN_TRUE		RETURN_BOOL(1)
#define RETURN_FALSE	RETURN_BOOL(0)

#define MAX_DIMENSION	100

// --------------------------------------------------------------------

SEXP R_initialize_benchmark (SEXP _fid, SEXP _dim, SEXP _seed) {
	int fid, dim, seed;

	if (!IS_INTEGER(_fid))
		Rf_error("the function id has to be integer");
	if (!IS_INTEGER(_dim))
		Rf_error("the number of dimensions has to be given as an integer number");
	if (!IS_INTEGER(_seed))
		Rf_error("the seed value has to be given as an integer number");

	fid  = INTEGER_DATA(_fid)[0];
	dim  = INTEGER_DATA(_dim)[0];
	seed = INTEGER_DATA(_seed)[0];

	set_input_data_dir(input_data_path);
	initialize_benchmark(fid, dim, seed);

	RETURN_TRUE
}

// --------------------------------------------------------------------

SEXP R_destroy_benchmark () {
	RETURN_BOOL(destroy_benchmark())
}

// --------------------------------------------------------------------

SEXP R_compute_benchmark (SEXP _x) {
	double * x, value;
	unsigned int dim;
	SEXP ret;

	if (!IS_NUMERIC(_x))
		Rf_error("evaluation point has to be a numeric vector");

	x   = NUMERIC_DATA(_x);
	dim = LENGTH(_x);

	value = evaluate_benchmark (x, dim);

	PROTECT(ret = NEW_NUMERIC(1));
	NUMERIC_DATA(ret)[0] = value;
	UNPROTECT(1);
	return ret;
}

// --------------------------------------------------------------------

SEXP R_get_optimum_value () {
	SEXP ret = PROTECT(NEW_NUMERIC(1));
	NUMERIC_DATA(ret)[0] = get_optimum_value();
	UNPROTECT(1);
	return ret;
}

// --------------------------------------------------------------------

SEXP R_get_accuracy () {
	SEXP ret = PROTECT(NEW_NUMERIC(1));
	NUMERIC_DATA(ret)[0] = get_accuracy();
	UNPROTECT(1);
	return ret;
}

// --------------------------------------------------------------------

SEXP R_get_init_range () {
	SEXP ret = PROTECT(NEW_LIST(2));
	SEXP low = PROTECT(NEW_NUMERIC(get_init_range_size()));
	SEXP upp = PROTECT(NEW_NUMERIC(get_init_range_size()));
	get_init_range(NUMERIC_DATA(low), NUMERIC_DATA(upp), get_init_range_size());
	SET_ELEMENT(ret, 0, low);
	SET_ELEMENT(ret, 1, upp);

	SEXP nms = PROTECT(NEW_CHARACTER(2));
	SET_STRING_ELT(nms, 0, mkChar("lower"));
	SET_STRING_ELT(nms, 1, mkChar("upper"));
	SET_NAMES(ret, nms);

	UNPROTECT(4);
	return ret;
}

// --------------------------------------------------------------------

SEXP R_get_search_range () {
	if (get_search_range_size() <= 0)
		return R_NilValue;

	SEXP ret = PROTECT(NEW_LIST(2));
	SEXP low = PROTECT(NEW_NUMERIC(get_search_range_size()));
	SEXP upp = PROTECT(NEW_NUMERIC(get_search_range_size()));
	get_search_range(NUMERIC_DATA(low), NUMERIC_DATA(upp), get_search_range_size());
	SET_ELEMENT(ret, 0, low);
	SET_ELEMENT(ret, 1, upp);

	SEXP nms = PROTECT(NEW_CHARACTER(2));
	SET_STRING_ELT(nms, 0, mkChar("lower"));
	SET_STRING_ELT(nms, 1, mkChar("upper"));
	SET_NAMES(ret, nms);

	UNPROTECT(4);
	return ret;
}

// --------------------------------------------------------------------

SEXP R_set_input_data_path (SEXP _path) {
	if (!IS_CHARACTER(_path) || LENGTH(_path) != 1)
		Rf_error("`path' is expected to be one-element character vector");
	strncpy(input_data_path, CHAR(STRING_ELT(_path, 0)), 1024);
	RETURN_TRUE;
}

// --------------------------------------------------------------------

R_CallMethodDef callMethods[] = {
	{"initialize_benchmark", (DL_FUNC) & R_initialize_benchmark, 3},
	{"destroy_benchmark",    (DL_FUNC) & R_destroy_benchmark, 0},
	{"compute_benchmark",    (DL_FUNC) & R_compute_benchmark, 1},
	{"get_optimum_value",    (DL_FUNC) & R_get_optimum_value, 0},
	{"get_acccuracy",        (DL_FUNC) & R_get_accuracy, 0},
	{"get_init_range",       (DL_FUNC) & R_get_init_range, 0},
	{"get_search_range",     (DL_FUNC) & R_get_search_range, 0},
	{"set_input_data_path",  (DL_FUNC) & R_set_input_data_path, 1},
	{NULL, NULL, 0}
};

// --------------------------------------------------------------------

void R_init_cec05testbed (DllInfo *info) {
    report_error_ptr = Rf_error;
    R_registerRoutines(info, NULL, callMethods, NULL, NULL);
}

// --------------------------------------------------------------------

void R_unload_cec05testbed (DllInfo * info) {
	destroy_benchmark();
}

