// vim: set syntax=cpp :

/** @file task.hpp
 *
 * @brief Task class template.
 *
 * Created on: 2010-02-10
 * Copyright (C) by Łukasz Bartnik
 */

#include <iostream>
#include <fstream>
#include <limits>
#include <list>
#include <vector>
#include <string>
#include <sstream>
#include <cstring>

#include <cstdio>
#include <cstdlib>
#include <cmath>

#include "wrapper.h"

// ---------------------------------------------------------------------

struct Task {
	virtual double evaluate (const double * _point, unsigned int _size) = 0;
	virtual unsigned int getTaskNo () const = 0;
};

// ---------------------------------------------------------------------

namespace __task_namespace_f1 {

	#define f1
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<1>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f1

}

// ---------------------------------------------------------------------

namespace __task_namespace_f2 {

	#define f2
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<2>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f2

}

// ---------------------------------------------------------------------

namespace __task_namespace_f3 {

	#define f3
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<3>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f3

}

// ---------------------------------------------------------------------

namespace __task_namespace_f4 {

	#define f4
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<4>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f4

}

// ---------------------------------------------------------------------

namespace __task_namespace_f5 {

	#define f5
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<5>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f5

}

// ---------------------------------------------------------------------

namespace __task_namespace_f6 {

	#define f6
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<6>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f6

}

// ---------------------------------------------------------------------

namespace __task_namespace_f7 {

	#define f7
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<7>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f7

}

// ---------------------------------------------------------------------

namespace __task_namespace_f8 {

	#define f8
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<8>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f8

}

// ---------------------------------------------------------------------

namespace __task_namespace_f9 {

	#define f9
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<9>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f9

}

// ---------------------------------------------------------------------

namespace __task_namespace_f10 {

	#define f10
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<10>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f10

}

// ---------------------------------------------------------------------

namespace __task_namespace_f11 {

	#define f11
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<11>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f11

}

// ---------------------------------------------------------------------

namespace __task_namespace_f12 {

	#define f12
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<12>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f12

}

// ---------------------------------------------------------------------

namespace __task_namespace_f13 {

	#define f13
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<13>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f13

}

// ---------------------------------------------------------------------

namespace __task_namespace_f14 {

	#define f14
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<14>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f14

}

// ---------------------------------------------------------------------

namespace __task_namespace_f15 {

	#define f15
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<15>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f15

}

// ---------------------------------------------------------------------

namespace __task_namespace_f16 {

	#define f16
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<16>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f16

}

// ---------------------------------------------------------------------

namespace __task_namespace_f17 {

	#define f17
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<17>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f17

}

// ---------------------------------------------------------------------

namespace __task_namespace_f18 {

	#define f18
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<18>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f18

}

// ---------------------------------------------------------------------

namespace __task_namespace_f19 {

	#define f19
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<19>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f19

}

// ---------------------------------------------------------------------

namespace __task_namespace_f20 {

	#define f20
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<20>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f20

}

// ---------------------------------------------------------------------

namespace __task_namespace_f21 {

	#define f21
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<21>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f21

}

// ---------------------------------------------------------------------

namespace __task_namespace_f22 {

	#define f22
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<22>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f22

}

// ---------------------------------------------------------------------

namespace __task_namespace_f23 {

	#define f23
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<23>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f23

}

// ---------------------------------------------------------------------

namespace __task_namespace_f24 {

	#define f24
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<24>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f24

}

// ---------------------------------------------------------------------

namespace __task_namespace_f25 {

	#define f25
	#include "testbed/rand.h"
	#include "testbed/sub.h"
	#include "testbed/global.h"
	#include "testbed/auxiliary.c"
	#include "testbed/def1.c"
	#include "testbed/def2.c"
	#include "testbed/def3.c"
	#include "testbed/def4.c"
	#include "testbed/rand.c"
	#include "task.tpp"
	Task * get_instance (unsigned int _dim, unsigned int _seed, const char * _dpath) {
		return new TaskTpl<25>(_dim, _seed, _dpath, nfunc, nreal, seed);
	}
	#undef _GLOBAL_H
	#undef _RAND_H
	#undef _SUB_H
	#undef f25

}

// vim: set syntax=cpp :

// optimal values
long double optima_values[25] = {
	-450, -450, -450, -450, -310,
	 390, -180, -140, -330, -330,
	  90, -460, -130, -300,  120,
	 120,  120,   10,   10,   10,
	 360,  360,  360,  260,  260
};

// accuracy criterion
long double accuracy[25] = {
	1e-6, 1e-6, 1e-6, 1e-6, 1e-6,
	1e-2, 1e-2, 1e-2, 1e-2, 1e-2,
	1e-2, 1e-2, 1e-2, 1e-2, 1e-2,
	1e-2, 1e-1, 1e-1, 1e-1, 1e-1,
	1e-1, 1e-1, 1e-1, 1e-1, 1e-1
};

double lower_bound[25] = {
	-100, -100, -100, -100, -100,
	-100,    0,  -32,   -5,   -5,
	-0.5,-M_PI,   -5, -100,   -5,
	  -5,   -5,   -5,   -5,   -5,
	  -5,   -5,   -5,   -5,    2
};

double upper_bound[25] = {
	 100,  100,  100,  100,  100,
	 100,  600,   32,    5,    5,
	 0.5, M_PI,    5,  100,    5,
	   5,    5,    5,    5,    5,
	   5,    5,    5,    5,    5
};

// ---------------------------------------------------------------------

#include <boost/preprocessor/repetition/repeat_from_to.hpp>
#include <cstdarg>

static Task * task_ptr = NULL;
error_func_ptr report_error_ptr = report_error;
static unsigned int dimensions_no = 0;
static char input_data_path[1024] = {0};

int initialize_benchmark (int _no, unsigned int _dim, unsigned int _seed) {
	if (task_ptr) {
		report_error_ptr("already initialized: call `destroy' first");
		return false;
	}
	if (_dim != 2 && _dim != 10 && _dim != 30 && _dim != 50) {
		report_error_ptr("Wrong value of dimensions given: only 2, 10, 30, 50 variables are supported");
		return false;
	}

	#define CASE__(z,no,data) case no: task_ptr = __task_namespace_f ## no ::get_instance(_dim,_seed,input_data_path); break;
	switch(_no) {
		BOOST_PP_REPEAT_FROM_TO(1,
	        25,
			CASE__,
			_);
		default:
			report_error_ptr("unknown function id: %d", _no);
			return false;
	}
	#undef CASE__

	dimensions_no = _dim;
}

double evaluate_benchmark (const double * _point, unsigned int _size) {
	if (!task_ptr) {
		report_error_ptr("not initialized: call `initialize' first");
		return std::numeric_limits<double>::quiet_NaN();
	}

	return task_ptr->evaluate(_point, _size);
}

int destroy_benchmark () {
	if (!task_ptr) {
		report_error_ptr("not initialized: call `initialize' first");
		return false;
	}

	delete task_ptr;
	task_ptr = NULL;

	return true;
}

void report_error (const char * fmt, ...) {
	va_list ap;
	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
	exit(EXIT_FAILURE);
}

long double get_optimum_value () {
	if (!task_ptr) {
		report_error_ptr("not initialized: call `initialize' first");
		return std::numeric_limits<long double>::quiet_NaN();
	}
	return optima_values[task_ptr->getTaskNo()-1];
}

long double get_accuracy () {
	if (!task_ptr) {
		report_error_ptr("not initialized: call `initialize' first");
		return std::numeric_limits<long double>::quiet_NaN();
	}
	return accuracy[task_ptr->getTaskNo()-1];
}

unsigned int get_init_range_size () {
	if (!task_ptr) {
		report_error_ptr("not initialized: call `initialize' first");
		return 0xDEADBEEF;
	}
	return dimensions_no;
}

unsigned int get_search_range_size () {
	if (!task_ptr) {
		report_error_ptr("not initialized: call `initialize' first");
		return 0xDEADBEEF;
	}
	if (task_ptr->getTaskNo() == 7 || task_ptr->getTaskNo() == 25)
		return 0;
	else
		return dimensions_no;
}

void get_init_range (double * _lower, double * _upper, unsigned int _size) {
	if (!task_ptr) {
		report_error_ptr("not initialized: call `initialize' first");
		return;
	}
	if (_size != get_init_range_size()) {
		report_error_ptr("output buffer size not equal to init-range-size");
		return;
	}

	for (unsigned int i=0; i<_size; ++i) {
		_lower[i] = lower_bound[task_ptr->getTaskNo()-1];
		_upper[i] = upper_bound[task_ptr->getTaskNo()-1];
	}
}

void get_search_range (double * _lower, double * _upper, unsigned int _size) {
	if (!task_ptr) {
		report_error_ptr("not initialized: call `initialize' first");
		return;
	}
	if (_size != get_init_range_size()) {
		report_error_ptr("output buffer size not equal to search-range-size");
		return;
	}

	for (unsigned int i=0; i<_size; ++i) {
		_lower[i] = lower_bound[task_ptr->getTaskNo()-1];
		_upper[i] = upper_bound[task_ptr->getTaskNo()-1];
	}
}

void set_input_data_dir (const char * _path) {
	strncpy(input_data_path, _path, 1024);
}

